local beam = {}
beam.__index = beam
local function new(joints,metal,building,loadType)
    local x={}
    local y={}

    for i=1,2 do
        if type(joints[i].body)=="userdata" then
            x[i]=joints[i].body:getX()
            y[i]=joints[i].body:getY()
        else
            x[i]=joints[i][1]
            y[i]=joints[i][2]

        end
    end
    local mass=math.round(math.sqrt((x[2]-x[1])^2+(y[2]-y[1])^2)*metal.mass*building.mass)
    mainBase:useSelectedMetal(metal,mass*building.cost)
    local tStrength=metal.tensile*building.strength
    local cStrength=metal.compressive*building.strength
    local young=metal.young*building.strength
    if building.name=='Cable' or building.name=='Heavy Cable' then
       --young=young*building.strength
    end
    local dColour=metal.colour
    local cColour={0,255,0}
    local node={}
    for i=1,2 do
        if type(joints[i].body)=="userdata" then
            if not loadType  and joints[i].mobile=='dynamic' then
                joints[i]:increaseMass(mass/2)
            end
            node[i]=joints[i]
        else
            node[i]=joint(joints[i][1],joints[i][2],mass/2,joints[i][3])
            table.insert(brg.joints,node[i])
        end
    end
    if x[1]>x[2] then
       local tempN=node[2]
       node[2]=node[1]
       node[1]=tempN
    end
    local x1,y1,x2,y2=node[1].body:getX(),node[1].body:getY(),node[2].body:getX(),node[2].body:getY()
    local j={}

    local rLength=0
    node[3]={}
    local jointCable={}
    local length=mag(x1-x2,y1-y2)
    if building.name=='Cable' or building.name=='Heavy Cable' then

       local midX=(x1+x2)/2
       local midY=(y1+y2)/2
       node[3]=joint(midX,midY,mass)
       rLength=length
       jointCable=love.physics.newDistanceJoint(node[1].body,node[3].body,x1,y1,midX,midY,false)
       j=love.physics.newDistanceJoint(node[3].body,node[2].body,midX,midY,x2,y2,false)
    else
       j=love.physics.newDistanceJoint(node[1].body,node[2].body,x1,y1,x2,y2,false)
    end
    local width=building.drawWidth
    return setmetatable({jointCable=jointCable,df=15,active=true,width=width,metal=metal,building=building,j1=node[1],j2=node[2],j3=node[3],mass=mass,joint=j,force=5,oldForce=0,cStrength=cStrength,tStrength=tStrength,young=young,cColour=cColour,rope=rLength,dColour=dColour,length=length}, beam)
end

function beam:draw()
    if brg.showForces then
        love.graphics.setColor(self.cColour[1],self.cColour[2],self.cColour[3])
    else
        love.graphics.setColor(self.dColour[1],self.dColour[2],self.dColour[3])
    end
    love.graphics.setLineWidth(self.width)
    if self.active==false then
        return
    end
    if self.rope>0 then
       local x1,y1,x2,y2,x3,y3=self.j1.body:getX(),self.j1.body:getY(),self.j2.body:getX(),self.j2.body:getY(),self.j3.body:getX(),self.j3.body:getY()
           love.graphics.line(x1,y1,x3,y3)
           love.graphics.line(x3,y3,x2,y2)
    else
        local x1,y1,x2,y2=self.joint:getAnchors()
        love.graphics.line(x1,y1,x2,y2)
        if self.building.name=='Deck' then
            love.graphics.setColor(50,50,50)
            love.graphics.line(x1,y1-4,x2,y2-4)
        end
    end
    --love.graphics.print(self.force,(x2+x1)/2,(y2+y1)/2)
end

function beam:update(dt)
    if self.active==false then
        return
    end
    local x1,y1,x2,y2=self.joint:getAnchors()
    --if y1>1200 then
    --   self:destroy()
    --end
    local xForce,yForce=self.joint:getReactionForce(dt)
    xForce=xForce*60
    yForce=yForce*60
    local sign=1
    if (xForce==0) then
        sign=math.sign(self.j2.body:getY()-self.j1.body:getY())*math.sign(yForce)
    else
        sign=math.sign(self.j2.body:getX()-self.j1.body:getX())*math.sign(xForce)
    end
    self.force=math.floor((self.oldForce*15+mag(xForce,yForce)*sign)/16)
    --self.force=math.floor(constrain(mag(xForce,yForce)*sign,-self.df-math.abs(self.oldForce),self.df+math.abs(self.oldForce)))
    self.oldForce=self.force
    if (self.force<0) then
       if self.force<-self.tStrength then
           self:destroy()
           return
       end
    elseif (self.force>0) then
       if self.force>self.cStrength then
           self:destroy()
           return
       end
    end
    if gi.selectedMenu==2 then
        local r=0
        if (self.force<0) then
            r=math.abs(self.force/self.tStrength)*255
            self.cColour={r,150,0}
        else
            r=math.abs(self.force/self.cStrength)*255
           self.cColour={r,0,150}
        end
    end
    local rMod=0
    local newLength=0
    --local testLength=mag(self.j1.body:getX()-self.j1.body:getY(),self.j2.body:getX()-self.j2.body:getY())
        newLength=math.round((-self.force*self.length)/self.young+self.length)

    --elseif self.force>10 and self.rope>testLength then        --self.rope>testLength and
    --    newLength=testLength-0.1
    --    self.force=-1000
    --elseif self.rope>testLength and self.force<0 then
    --    newLength=testLength+0.5
    --elseif self.force<0 and self.rope<=testLength then
    --    newLength=math.round((-self.force*self.length)/self.young+self.length)
    --end
    if self.rope>0 then
        self.jointCable:setLength(newLength/2)
        self.joint:setLength(newLength/2)
    else
        self.joint:setLength(newLength)
    end
end
function beam:destroy()
    self.active=false
    table.deleteObject(brg.beams,self)
    mainBase:useSelectedMetal(self.metal,-self.mass)
    self.joint:destroy()
end


return setmetatable({new = new},
    {__call = function(_, ...) return new(...) end})