local building = {}
building.__index = building
local function new(type,load)
    local b,placed,x,y,active,speed,activeRow,connected,input,output
    local img=love.graphics.newImage("building"..type..".png")
    if load then
        type=load.type
        b=load.b
        placed=load.placed
        x=load.x
        y=load.y
        active=load.active
        speed=load.speed
        activeRow=load.activeRow
        connected=load.connected
        input=load.input
        output=load.output
    else
        type=type
        b=buildingTypes[type]
        placed=false
        x=0
        y=0
        active=false
        speed=b.speed
        activeRow=-1
        connected=false
        input={}
        output={}
    end
    local depth=b.place[1]
    local direction=b.place[2]
    return setmetatable({b=b,selected=true,canPlace=false,input=input,output=output,x=x,y=y,width=img:getWidth(),height=img:getHeight(),type=type,placed=placed,img=img,joint=-1,active=active,speed=speed,ticker=0,activeRow=activeRow,connected=connected,direction=direction,depth=depth}, building)
end
function building:save()
    return {type=self.type,b=table.copy(self.b),placed=self.placed,x=self.x,y=self.y,active=self.active,speed=self.speed,activeRow=self.activeRow,connected=self.connected}
end
function building:update(dt)
    if self.placed==false then
            gi.cursor='building'
            self.x,self.y,self.canPlace=self:findFirstBuildable()
    else
        self.ticker=self.ticker+dt*50
        if (self.ticker>self.speed) then
            if self.connected then
                self.active=true
            else
                self.active=false
            end
            self.ticker=0
            if self.type==1 and self.active then
                self:shaftDig()
            elseif self.type==4 and self.active then
                self:radiusDig()
            end
        end

    end
end

function building:draw()
    --love.graphics.translate(-12,0)
    if self.placed then
        love.graphics.setColor(255,255,255,255)
    else
        if self.canPlace~=false then
            love.graphics.setColor(50,255,50,150)
        else
            love.graphics.setColor(255,50,50,150)


        end
    end
    if self.type==4 and self.selected then
        love.graphics.rectangle('line',self.x-self.b.maxDistance*4,self.y-self.b.maxDistance*4,self.b.maxDistance*8,self.b.maxDistance*8)
    end
    love.graphics.draw(self.img,self.x,self.y)

    if (self.type>=102 and self.type<=106 and self.joint~=-1) then
        if (self.canPlace==false) then
            love.graphics.setColor(50,255,50,150)
        else
            love.graphics.setColor(230,255,50,150)
        end
        love.graphics.line(self.joint.body:getX(),self.joint.body:getY(),self.x,self.y);
     end
     --love.graphics.translate(12,0)
end
function building:moveSide(direction)
end

function building:moveUp()
end

function building:openDialog()
    gi:openDialog({
        {'sub',self.b.name},
        {'textDesc',self.b.description},
        {'sub','Resources needed'},
        {'text','Energy',self.b.energy},
        {'text','Material',self.b.material},
        {'sub','Setup'},
        {'functionButton','Ok',function()
        gi:closeDialog()
        self.selected=false
        self:placeBuilding()
        end},
    })
    --self.currentSubject.selected=false
    --if buildingToPlace==self.currentSubject then
      --  self.currentSubject:cancelPlacement()
    --end
end
function building:place()
    if self.placed==true then
        return
    end
    if self.type>=102 and self.type<=106 then
        if self.joint==-1 and self.canPlace~=false then
            self.joint=self.canPlace
        elseif self.joint~=-1 and self.canPlace~=false then
            if (self.joint.mobile=='static' and self.canPlace.mobile=='static') then
                gi:logText('Cannot connect beam to 2 anchors.','bridge')
                return
            elseif (self.joint==self.canPlace) then
                return
            end
            local beamy=brg:getConnectedToJoints(self.canPlace,self.joint)
            if  beamy~=false then
                gi:logText('Replaced beam.','bridge')
                beamy:destroy()
            end
            local beamster=beam({self.joint,self.canPlace},mainBase:getSelectedMetal(),self.b)

            self.canPlace=-1
            self.joint=-1
            table.insert(brg.beams,beamster)
        elseif self.joint~=-1 then
            local beamster=beam({{self.x,self.y,false},self.joint},mainBase:getSelectedMetal(),self.b)
            self.canPlace=-1
            self.joint=-1
            table.insert(brg.beams,beamster)
        end
        return
    elseif self.type==107 then
        if self.canPlace then
            brg:deleteWithJoint(self.canPlace)
            self.canPlace=-1
        end
        return
    end
    if self.canPlace then
        local checkC=mainBase:checkCost(self.b.material,self.b.energy)
        if not checkC then
            return
        end
        gi.cursor='normal'
        self.placed=true;
        self:openDialog()
    end
end
function building:placeBuilding()
    if buildingToPlace==self then

        buildingToPlace=-1
        self.selected=false
        mainBase:connect(self)
        if (self.type==1) then
            self.activeRow=math.floor(self.y/4)
            for i=0,math.floor(self.width/4)-1 do
                tileX,tileY=toGrid(self.x,self.y)
                currentMap:setPoint(tileX+i,tileY+3,252)
            end
        elseif (self.type==101) then
            self.joint=joint(self.x+8,self.y+8,0,true)
            table.insert(brg.joints,self.joint)
        elseif self.type==5 then
            local lightTest=light(self.x,self.y,150,self.b.lightStrength)
            lightTest:draw()
            table.insert(objects.lights,lightTest)
            blendLights()
        end
        table.insert(objects.buildings,self)
    end
end

function building:cancelPlacement()
    buildingToPlace=-1
    self.placed=false
end
function building:dialogCallback(call)
   local tempAr={}
   if call=="inputDirt" then
        tempAr[1]=true
        lookingForConnection={'output',tempAr,self}
        textBuffer='Choose input shaft'
       text='choose'
   end

end
function building:shaftDig()
    local tileX=math.floor(self.x/4)
    local tileY=self.activeRow
    if currentMap:isSolid(currentMap:checkPoint(self.x-4,self.y)) then
        self.active=false
        return
    end
    local type=255
    for i=0,math.floor(self.width/4)-1 do
        type=currentMap:checkPoint(tileX+i,tileY,1)
        if type<240 then
            currentMap:setPoint(tileX+i,tileY,255)
            if i%math.round(self.b.crush)==0 then
                if next(self.output)==nil then
                    table.insert(objects.dirt,dirt(tileX*4,math.floor(self.y/4)*4,type,-1))
                else
                    table.insert(objects.dirt,dirt(self.output[1].x,self.output[1].y,type,-1))
                end
            end
            return
        end


    end
    if (self.activeRow-math.floor(self.y/4))%16==0 then
        local lightTest=light(tileX*4+self.width/2,self.activeRow*4,100,self.b.lightStrength)
        lightTest:draw()
        table.insert(objects.lights,lightTest)
        blendLights()
    end
    currentMap:setPoint(tileX,tileY,253)
    self.activeRow=self.activeRow+1
end
function building:radiusDig()
    local surrounded=self.activeRow*8
    for i=1,surrounded do
        local tX,tY=toGrid(self.x+self.width/2-1,self.y)
        local iMod=-math.abs(i-1-self.activeRow*3)+self.activeRow*3
        local i2=((i+(surrounded-self.activeRow*2-1))%surrounded)+1
        local iModY=-math.abs(i2-1-self.activeRow*3)+self.activeRow*3
        local xOffset=math.max(0,math.min(self.activeRow*2,iMod))-self.activeRow
        local yOffset=math.max(0,math.min(self.activeRow*2,iModY))-self.activeRow
        local type=currentMap:checkPoint(tX+xOffset,tY+yOffset,1)
        if currentMap:isOre(type) then
            currentMap:setPoint(tX+xOffset,tY+yOffset,255)
            table.insert(objects.dirt,dirt((tX+xOffset)*4,(tY+yOffset)*4,type,'dissolve',self))
            return
        end
    end
    if (self.activeRow<self.b.maxDistance) then
        self.activeRow=self.activeRow+1
    end

end

function building:clickActive()
    if lookingForConnection~=-1 then
        text='found'
        if lookingForConnection[2][self.type] then
            if lookingForConnection[1]=='input' then
                table.insert(self.input,lookingForConnection[3])
                table.insert(lookingForConnection[3].output,self)

            else
                table.insert(self.output,lookingForConnection[3])
                table.insert(lookingForConnection[3].input,self)
            end
            lookingForConnection=-1
        end
    else
        self.selected=true
        gi.currentSubject=self
        gi:openDialog({
            {'sub','Setup'},
            self.b.options
        })
    end
end
function building:findFirstBuildable()
    local tempXY=cam:worldCoords(love.mouse.getX(),love.mouse.getY())
    local x=tempXY.x
    local y=tempXY.y
if self.direction=='point' then
    local canPlace=false
    local slack=0
    x,y,canPlace=brg:findClosestJoint(x,y,12)
    if canPlace then
       slack=5
    end
    if self.joint~=-1 then
        if mag(x-self.joint.body:getX(),y-self.joint.body:getY())>self.b.maxLength+slack then
            canPlace=false
            x,y=getPointTowards(self.joint.body:getX(),self.joint.body:getY(),x,y,self.b.maxLength)
            --love.mouse.setPosition(x,y)
        end
    end
    return x,y,canPlace
elseif self.direction=='beam' then
    local canPlace=false
    x,y,canPlace=brg:findClosestBeam(x,y,12)
    return x,y,canPlace
end
local xTile,yTile,direction=currentMap:findFirstGround(x,y,self.direction)

local modX=0
local modY=0
local widthGrid=math.floor(self.width/4)

if xTile~=-1 then
    xTile=math.floor(xTile-widthGrid/2)
    if direction=='down' then
        modY=2
    else
        modX=2
    end
    local clear=true
    for i=0,widthGrid-1 do
        if not currentMap:isEmpty(currentMap:checkPoint(xTile+i,yTile-modY,1)) or currentMap:isEmpty(currentMap:checkPoint(xTile+i,yTile+modY,1)) then
            clear=false
            break
        end
    end
    if mainBase.y[1]-y>mainBase.controlDistance then
        clear=false
    end
    if clear==false then
        return x-math.floor(self.width/8)*4,y,false
    else
        if (direction=='down') then
            yTile=yTile+self.depth
        else

        end

        return  xTile*4,yTile*4,true

    end


else
    return x-math.floor(self.width/8)*4,y,false
end
end



return setmetatable({new = new},
    {__call = function(_, ...) return new(...) end})