constantGreen={255,0,0}
constantRed={255,0,0}
levelsMass={0.8,1.0,1.4,2.1,2.6,3.2,3.4}
levelsTensile={5000,5500,6000,7000,10000,12000,14000}
levelsCompressive={8000,9000,10000,11000,12000,14000,16000}
levelsYoung={290000,295000,300000,310000,320000,34000,360000 }
waterBody={}
eSource={
    {name='RTG',description='Radioisotope thermoelectric generators provide a small but reliable source of power.',energy=100,cost=1000,limit=1,have=1},
}
colours={dialogBackground={210,210,210,255},
         dialogOutline={50,50,50,255},
         dialogFrame={180,180,180,255},
         text={0,0,0,255},
         buttonBackground={170,170,170,255},
         white={255,255,255,255},
}
biomes={
    {{40,100,200},'hill'},
    {{20,50,300},'bumpy'},
    {{1,500,2000},'flat'},
}
veinDensity={{4,6},{3,8},{5,8},{5,8},{4,7},{4,7}}

function math.sign(x)
    if x<0 then
        return -1
    elseif x>0 then
        return 1
    else
        return 0
    end
end
function math.round(num, idp)
    local mult = 10^(idp or 0)
    return math.floor(num * mult + 0.5) / mult
end
function table.deleteObject(tab,obj)
    for i,v in ipairs(tab) do
        if v==obj then
            table.remove(tab,i)
            return
        end
    end
end
function inTable(tab,obj)
    for i,v in ipairs(tab) do
        if v==obj then
            return true
        end
    end
    return false
end
function getPointTowards(x1,y1,x2,y2,distance)
    local ang=math.atan2(y2-y1,x2-x1)
    local x=x1+distance*math.cos(ang)
    local y=y1+distance*math.sin(ang)
    return x,y
end

function generateWord(min,syl,type)
    local c  = 'bcdfghjklmnprstvwz'
    local v  = 'aeiou'
    local a  = c..v
    local pw=''

    for j=1,syl do
        local rand=math.random(1, string.len(c))
        pw =pw..c:sub(rand,rand);

        local rand=math.random(1, string.len(v))
        pw =pw..v:sub(rand,rand);

        if (math.random(1,10)>3 or syl==1) then
            local rand=math.random(1, string.len(a))
            pw =pw..a:sub(rand,rand);
        end
        if (syl==1) then
            local rand=math.random(1, string.len(v))
            pw =pw..v:sub(rand,rand);
        end
        if (math.random(1,10)>7 and j>=min) then
            break
        end
    end
    if (type=='ore') then
        pw=pw..'ite'
    end
    return pw
end
function table.copy(t)
    local u = { }
    for k, v in pairs(t) do u[k] = v end
    return setmetatable(u, getmetatable(t))
end
function constrain(x,min,max)
    return math.min(max,math.max(x,min))
end


function updateObject(x,y,xSpeed,ySpeed,dt,bouyancy)
    local onGround=false;
    local inWater=0
    if currentMap:isWater(currentMap:checkPoint(x,y)) then
        inWater=1
    else
        inWater=0
    end
    if currentMap:isSolid(currentMap:checkPoint(x+xSpeed*dt,y+ySpeed*dt)) then
        onGround=true
        ySpeed=0
    else
        ySpeed=ySpeed+20-bouyancy*inWater
        onGround=false
    end
    if onGround then
        if xSpeed<0 then
            xSpeed=math.min(0,xSpeed+6)
        elseif xSpeed>0 then
            xSpeed=math.max(0,xSpeed-6)
        end
    end
    if x<5 then
        xSpeed=math.max(0,xSpeed)
    elseif x> world.width-5  then
        xSpeed=math.min(0,xSpeed)
    else
        if currentMap:isSolid(currentMap:checkPoint(x+4,y-8)) then
            xSpeed=math.min(0,xSpeed)
        end
        if currentMap:isSolid(currentMap:checkPoint(x-4,y-8)) then
            xSpeed=math.max(0,xSpeed)
        end
    end
    x=math.max(5,math.min(world.width,x+xSpeed*dt))
    y=math.max(5,math.min(world.height,y+ySpeed*dt))

    return x,y,xSpeed,ySpeed,onGround,inWater
end

function toGrid(x,y)
    return math.floor(x/4),math.floor(y/4)
end

function toWorld(x,y)
    return x*4,y*4
end

function mag(x,y)
    return math.sqrt(x*x+y*y)
end

