local map = {}
map.__index = map
local function new(level)
    local frameBuffer ={}
    local fbd = love.graphics.newCanvas(200,100);
    fbd:clear()
    local levelThickness=105
    local soilLevel={}
    local reward=0
    local seeds={math.random(-214748364,214748364),math.random(-214748364,214748364),math.random(-214748364,214748364),math.random(-214748364,214748364),math.random(-214748364,214748364),math.random(-214748364,214748364),math.random(-214748364,214748364),}
    return setmetatable({soilLevel=soilLevel,reward=reward,loaded=false,showPrevious=0,previous={},tiles={},lines={},seed=seeds,level=0,left={},right={},fb=frameBuffer,fbd=fbd,levelThickness=levelThickness,startX=1,endX=1024,wStart=0,wEnd=0}, map)
end
function map:getLevel(level)
   self.startX=1
   local border=256
   local waterWidth=(math.random(100,120)/100)*(level*15)+10
   local wat={}
   self.endX=math.floor(waterWidth+border*2)
   if math.random(1,10)>6 and level>2 then
      local percent=math.random(4,7)/10
      local wat1=math.floor(waterWidth*percent)
      local wat2=math.floor(waterWidth*(1-percent))
      wat= {{math.floor(self.endX/2-wat1),wat1},{math.floor(self.endX/2+wat2),wat2}}
   else
      wat= {{math.floor(self.endX/2),math.floor(waterWidth)}}
   end
       table.insert(waterBody,wat)
    wat['reward']=math.floor(waterWidth*(math.random(100,120)/100))
    return wat
end
function map:generateLines()
    for i=1,#materials.soils+1 do
        self.lines[i]=self:noiseLine(self.endX, biomes[3][1],self.seed[i])
    end
end
function map:generateBase(level)
    self.startX=1
    self.endX=1024
    self:generateLines()
    self:setupTiles()
    self:digAllVeins()
    self:drawMap()
    self.loaded=true
end

function map:generateLevel(waterBody)
    self.reward=waterBody['reward']
    self:generateLines()
    self:setupTiles(waterBody)
    self:fillWater(waterBody);
    self:drawMap()
    self.loaded=true
end
function map:getMissions(level)

    local result={}
    for g=1,2 do
        local results=self:getLevel(level);
        table.insert(result,results)
    end
    return result
end
function map:saveMaterials()
    local save={}
    save.metals={}
    save.ores={}
    save.metals={}
    for i,v in ipairs(materials.metals)do
        table.insert(save.metals,v:save())
    end
    for i,v in ipairs(materials.ores)do
        table.insert(save.ores,v:save())
    end
    for i,v in ipairs(materials.soils)do
        table.insert(save.metals,v:save())
    end
    return save
end
function map:unloadMap()
    objects.dirt={}
    local save={}
    save.buildings={}
    for i,v in ipairs(objects.buildings) do
        save.buildings[i]=v:save()
    end
    objects.buildings={}
    save.materials=self:saveMaterials()
    save.base=mainBase:save()
    writeSaveFile(save)
    return self
end
function map:drawMap()
    --Draw
    self.fb=love.graphics.newCanvas(math.floor((self.endX-self.startX+self.showPrevious)*4+4),world.height);
    self.lb=love.graphics.newCanvas(math.floor((self.endX-self.startX+self.showPrevious)*4+4),world.height);
    self.pb=love.graphics.newCanvas(math.floor((self.endX-self.startX+self.showPrevious)*4+4),world.height);
    self.nb=love.graphics.newCanvas(math.floor((self.endX-self.startX+self.showPrevious)*4+4),world.height);
    love.graphics.setCanvas(self.fb)
    for i=self.startX-self.showPrevious,self.endX do
        for j=1,math.floor(world.height/4),1 do
            self:drawPoint((i-self.startX+self.showPrevious)*4,j*4,mats[self.tiles[i][j]])
        end
    end
    love.graphics.setCanvas()
    love.graphics.setCanvas(self.pb)
    self.pb:clear()
    --self.soilLevel[0]=self.soilLevel[1]
    --self.soilLevel[#self.soilLevel+1]=self.soilLevel[#self.soilLevel]
    for i=self.startX-self.showPrevious,self.endX do
        for j=self.soilLevel[i],math.floor(world.height/4),1 do
            local n1=self.soilLevel[i-1] or self.soilLevel[i]
            local n2=self.soilLevel[i+1] or self.soilLevel[i]
            local normLevel=(self.soilLevel[i]+n1+n2)/3
            local alpha=math.min(0,(j-normLevel-34)*math.abs((j-normLevel-34)/6))
            love.graphics.setColor(0,0,0,255+alpha)
            love.graphics.point((i-self.startX+self.showPrevious)*4,j*4);
        end
    end
    love.graphics.setCanvas()
    blendLights()
end
function map:fillWater(waterBody)

    local wStart={}
    local wEnd={}
    for i,v in ipairs(waterBody) do
        wStart[i]=math.floor(v[1]-v[2]*3/4)
        wEnd[i]=math.floor(v[1]+v[2]*3/4)
    end
    local waterLevel=0
    for i=1,#wStart do
        waterLevel=math.max(self.soilLevel[wStart[i]],self.soilLevel[wEnd[i]])
        for j=wStart[i],wEnd[i] do
            for d=waterLevel,math.floor(world.height/4) do
                if self.tiles[j][d]==255 then
                    self.tiles[j][d]=254
                end
            end
        end
    end
    self.wStart=math.floor(wStart[1]*4)
    self.wEnd=math.floor(wEnd[#wEnd]*4)
end
function map:digAllVeins()
    --dig out ore veins
    local width=self.endX-self.startX
    local repeatTimes=math.floor(4*width/1024)
    local wWidth=math.floor((width*4)/repeatTimes)
    local ore={}
    for i,v in ipairs(veinDensity) do
        local noV=math.random(v[1],v[2])
        for j=1,noV do
            for reps=1,6 do
                ore=materials.ores[math.random(1,#materials.ores)]
                if (ore.soilType==i) then
                    break
                end
            end
            for section=1,repeatTimes do
                self:digVein(self.startX*4+math.random(math.floor(wWidth*(section-1)+10),math.floor(self.startX*4+wWidth*section-10)),math.random(math.floor((i-1)*self.levelThickness*4+500),math.floor(i*self.levelThickness*4+500)),ore)
            end
        end
    end
end
function map:setupTiles(waterBody)

    local width=self.endX-self.startX
    local currentTile=255
    local currentLevel=1
    if waterBody then
        for i,v in ipairs(waterBody) do
            for j=v[1]-v[2],v[1]+v[2] do

                local varX=(j-v[1]+v[2])/(v[2]*2)*2*math.pi-math.pi/2
                if type(self.lines[1][j])=='number' then
                    print (111)
                    self.lines[1][j]=((math.sin(varX)+1)*v[2])/300+self.lines[1][j]
                end
            end
        end
    end
    local waterLevel=100
    for i=self.startX,self.endX,1 do
        self.tiles[i]={}
        currentLevel=1
        for j=1,math.floor(world.height/4) do
            local nextLevelLine=math.round(currentLevel*self.levelThickness+self.lines[currentLevel][i]*150)+60
            if j>=nextLevelLine then
                if currentLevel==1 then
                    self.soilLevel[i]=j
                end

                currentLevel=math.min(currentLevel+1,7)
                if currentLevel==2 then
                    self.tiles[i][j]=1
                else
                    self.tiles[i][j]=currentLevel+20
                end
            elseif currentLevel==1 then
                self.tiles[i][j]=255
            else
                self.tiles[i][j]=currentLevel+20
            end
        end
    end
end
function map:digVein(x,y,ore)
love.graphics.setCanvas(self.fbd)
self.fbd:clear()
love.graphics.setLineStyle('smooth')
love.graphics.setColor(255,255,255,255)

local tempLine={}

for sp=1,math.random(1,3) do
    local tX=100
    local tY=50
    for j=1,math.random(2,4) do
        love.graphics.setLineWidth(math.random(2,8))
        local dtX=math.random(-12,12)
        local dtY=math.random(-6,6)
        for i=1,math.random(4,10) do
            table.insert(tempLine,tX)
            table.insert(tempLine,tY)
            dtX=dtX+math.random(-4,4)
            dtY=dtY+math.random(-4,4)
            tX=tX+dtX
            tY=tY+dtY
        end
        love.graphics.line(tempLine)
    end
end
    love.graphics.setCanvas()
local img=self.fbd:getImageData()

for i=1,199 do
   for j=1,99 do
        local r=img:getPixel(i,j)
        local tempX=math.floor((x+i-60)/4)
        local tempY=math.floor((y+j-30)/4)
       if r>250 and tempY<(world.height/4) and type (self.tiles[tempX])=='table' then
           if self.tiles[tempX][tempY]<240 then
               self.tiles[tempX][tempY]=ore.tileType
           end
       end
   end
end

end

function map:drawPoint(x,y,colour)
    love.graphics.setPoint(4, "rough")
    love.graphics.setColor(colour[1],colour[2],colour[3],255)
    love.graphics.point(x+2,y+2);
end

function map:draw()
    love.graphics.setColor(255,255,255,255)
    love.graphics.draw(self.fb,(self.startX-self.showPrevious)*4,0)
    if not debugOn then
        love.graphics.draw(self.lb,(self.startX-self.showPrevious)*4,0)
    end
end

function map:checkPoint(x,y,toG)
    if not toG then
        x,y=toGrid(x,y)
    end
    if x>self.startX-self.showPrevious and y>1 and x<self.endX and y<=math.floor(world.height/4) then
        return self.tiles[x][y]
    else
        return 0
    end
end
function map:setPoint(tX,tY,type)
    self.tiles[tX][tY]=type
    love.graphics.setCanvas(self.fb)
    self:drawPoint((tX-self.startX+self.showPrevious)*4,tY*4,mats[type])
    love.graphics.setCanvas()
end

function map:isSolid(n)
    return n<240 or n==252
end
function map:isWater(n)
    return n==254
end
function map:isEmpty(n)
    return n==255
end
function map:isMineral(n)
    return n<=16 and n>=5
end
function map:isOre(n)
    return n<240 and n>=28
end

function map:findFirstGround(x,y,direction)
    local tX,tY=toGrid(x,y)
    for i=-5,10 do
        if (direction=='down') and self:isSolid(self:checkPoint(tX,tY+i,1)) then
                return tX,tY+i,direction
        end
    end
    return -1,-1,-1
end

function map:noiseLine(n, granulation, seed)
    local function lerp(x,y,t) return x+(y-x)*t end		-- Linear interpolation: Faster, but uglier
    local function cerp(x,y,t) local f=(1-math.cos(t*math.pi))*.5 return x*(1-f)+y*f end		-- Cosine interpolation: Slower, but prettier
    math.randomseed(seed)
    local a=math.random()
    a=math.random()
    granulation = granulation or {2,8,32}
    local noise = {}

    for scalei,scale in ipairs(granulation) do
        local t = {}
        for i=1,n/scale+scale+1 do t[i] = math.random()-.5 end

        for i=1,n do
            local x,y,t = t[math.floor(i/scale)+1], t[math.floor(i/scale)+2], (i%scale)/scale
            noise[i] = (noise[i] or 0) * ((granulation[scalei-1] or 1) / scale) + cerp(x,y,t)
        end
    end
    return noise
end

return setmetatable({new = new},
    {__call = function(_, ...) return new(...) end})